import { ServiceLifetime, ServiceRegistry, getNameOfServiceType, } from "@wroud/di";
import { v4 as uuid } from "uuid";
export function getDependenciesGraph(serviceCollection) {
    const nodes = new Map();
    const links = [];
    function addUnknownNode(service) {
        const node = nodes.get(service);
        if (!node) {
            const id = uuid();
            const node = {
                id,
                name: getNameOfServiceType(service) || "Unknown",
                lifetime: ServiceLifetime.Transient,
                notFound: true,
            };
            nodes.set(service, node);
            return node;
        }
        return node;
    }
    function addDescriptorNode(descriptor, notFound) {
        const node = nodes.get(descriptor);
        if (!node) {
            const id = uuid();
            const node = {
                id,
                name: getNameOfServiceType(descriptor.implementation) ||
                    getNameOfServiceType(descriptor.service),
                lifetime: descriptor.lifetime,
                notFound,
            };
            nodes.set(descriptor, node);
            return node;
        }
        return node;
    }
    for (const descriptor of serviceCollection) {
        const source = addDescriptorNode(descriptor);
        const dependencies = ServiceRegistry.get(descriptor.implementation);
        if (!dependencies) {
            continue;
        }
        for (const dependency of dependencies.dependencies) {
            const service = Array.isArray(dependency) ? dependency[0] : dependency;
            const descriptors = serviceCollection.getDescriptors(service);
            if (descriptors.length === 0) {
                const target = addUnknownNode(service);
                links.push({
                    source: source.id,
                    target: target.id,
                    name: `${source.name} -> ${target.name}`,
                });
                continue;
            }
            for (const dep of descriptors) {
                const target = addDescriptorNode(dep);
                links.push({
                    source: source.id,
                    target: target.id,
                    name: `${source.name} -> ${target.name}`,
                });
            }
        }
    }
    return { nodes: Array.from(nodes.values()), links };
}
//# sourceMappingURL=getDependenciesGraph.js.map